#include "PCH.h"
#include "Explosion.h"
#include "Enemy.h"
#include "Level.h"
#include "gfx.h"
#include "sfx.h"

Explosion::Explosion(Level& level, float x, float y)
	: GameObject(level), x(x), y(y), radius(1.0f), angle(0.0f), scale(0.0f)
{
	m_iLayer = g_PlayerCannonDepth;
	g_hge->Effect_PlayEx(g_sndRocketExplosion, 100, calcPan(x), g_hge->Random_Float(0.75f, 1.1f));
}

Explosion::~Explosion()
{

}

void Explosion::update()
{
	if(radius > (float)c_Size ) dead = true;

	radius += 0.0025f * 25.0f;
	//radius *= 1.01f;
	angle += 0.01f;
	scale = 2.0f * radius / (float)c_Size;

	std::vector<Enemy*> enemiesHit = level.getEnemiesNear(x, y, radius + scale * 60.0f);
	if(enemiesHit.size())
	{
		for(std::vector<Enemy*>::iterator it = enemiesHit.begin();
			it != enemiesHit.end();
			it++)
		{
			(*it)->onHitByExplosion();
		}
	}
}

void Explosion::render()
{
	float alpha = 2.0f - 2.0f * (radius / (float)c_Size);
	if(alpha > 1.0f) alpha = 1.0f;
	else if(alpha < 0.0f) alpha = 0.0f;
	hgeColor color(1.0f, 1.0f, 1.0f, alpha * 0.25f);
	g_sprFlash->SetColor(color.GetHWColor());
	g_sprFlash->RenderEx(x, y, 0.0f, scale * 3.0f);

	for(int i = 0; i < 16; i++)
	{
		float angle = this->angle + M_PI * 2 * ((float)(i) / 16.0f);
		float px = x + sinf(angle) * radius;
		float py = y - cosf(angle) * radius;
		int idx = i%2;
		hgeColor color(1.0f, 1.0f, 1.0f, alpha);
		g_sprFlame[idx]->SetColor(color.GetHWColor());
		g_sprFlame[idx]->RenderEx(px, py, angle, scale);
	}
}