#include "PCH.h"
#include "Level.h"
#include "Field.h"
#include "GameObject.h"
#include "Enemy.h"
#include "Player.h"
#include "Text.h"
#include "gfx.h"
#include "sfx.h"
#include "TargetValue.h"

#include "EffectSprite.h"

#include <string>

Field* Level::GetLowestFromRow( int iRow, float& fLastPosY )
{
	for(int y = LEVEL_HEIGHT - 1; y >= 0; y--)
	{
		if( fields[iRow][y] != 0 )
		{
			Field* ret= fields[iRow][y];
			if( ret->type == 7 )
			{
				return NULL;
			}

			fields[iRow][y]= 0;
			fLastPosY= g_fBlockHeight * (float)y + m_fOffsetY;
			return ret;
		}
	}

	return NULL;
}

float Level::GetLowestPos( int iRow )
{
	for(int y = LEVEL_HEIGHT - 1; y >= 0; y--)
	{
		if( fields[iRow][y] != 0 )
		{
			return g_fBlockHeight * (float)y + m_fOffsetY;
		}
	}

	return 0.0f;
}

void Level::movePlayerDown( int iColor )
{
	float fActDepth= m_playerShips[ iColor ]->m_fDepth;
	for( int i= 0; i < 3; ++i )
	{
		if( i == iColor )
		{
			m_playerShips[ i ]->m_tvY.m_fValue-= 16.0f - fActDepth; 
			m_playerShips[ i ]->m_fDepth= 16.0f;
		}
		else if (m_playerShips[ i ]->m_fDepth > fActDepth )
		{
			m_playerShips[ i ]->m_tvY.m_fValue+= 8.0f; 
			m_playerShips[ i ]->m_fDepth-= 8.0f;
		}
	}
}

int Level::LowestPlayer( int iRow )
{
	float fDepth= -10.0f;
	int iRet= -1;
	for( int i= 0; i < 3; ++i )
	{
		if( m_playerShips[ i ]->m_iRow == iRow )
		{
			if( m_playerShips[ i ]->m_fDepth > fDepth )
			{
				fDepth= m_playerShips[ i ]->m_fDepth;
				iRet= i;
			}
		}
	}

	return iRet;
}

void Level::putBack( int iRow, Field* pField, float fPosY )
{
	int y;
	for( y = LEVEL_HEIGHT - 1; y >= 0; y--)
	{
		if( fields[iRow][y] != 0 )
		{
			break;
		}
	}
	y++;
	if( y == LEVEL_HEIGHT )
	{
		return;
	}

	fields[iRow][y]= pField;

	SHint hint;
	hint.m_iX= iRow;
	hint.m_iY= y;

	m_ActiveFields.push_back( hint );
	m_iTimeToCombo= c_iComboTime;

	pField->m_tvY.m_fValue+= fPosY - g_fBlockHeight * (float)y - m_fOffsetY;

	m_DownValue.m_fValue-= 2.0f;
}

void Level::addPoints( int iPoints, float x, float y )
{
	m_iPoints+= iPoints;

	if(x >= 0.0f && y >= 0.0f)
	{
		char str[256] = "";
		sprintf_s(str, "-%d-", iPoints);
		Text* text = new Text(*this, str, x, y, 0.5f, hgeColor(1.0f, 0.5f, 0.0f, 1.0f), 1.0f);
		adoptObject(text);
	}
}

void Level::addLevel( int iL )
{
	m_iSubStage+= iL;
	int iNewStage= m_iSubStage / 20;

	if( iNewStage > m_iStage )
	{
		m_iStage= iNewStage;
		m_fLevelUp= 2.5f;
		g_hge->Effect_PlayEx(g_sndLevel, 100, 0, 1.0f);
	}
}


void Level::checkCombo()
{
	int iLongestRow= 0;
	for(int x = 0; x < LEVEL_WIDTH; x++)
	{
		for(int y = 1; y < LEVEL_HEIGHT; y++)
		{
			if( fields[x][y] != NULL )
			{
				iLongestRow= max( iLongestRow, y );
			}
		}
	}

	if( iLongestRow < 12 )
	{
		newLine();
	}

	m_iResetCombo-= c_iTimeStep;
	if( m_iResetCombo <= 0 && m_iCombos > 0 )
	{
		addPoints( m_iCombos * m_iCombos * m_iStones, 512.0f, 100.0f );
		addLevel( m_iStones );
		m_StonesKilled+= m_iStones;

		if( m_iCombos > 1 )
		{
			addAmmo( m_iCombos / 2 );
		}

		if( m_iCombos > 2 &&  m_iStones > 4 )
		{
			StartGirl(m_iCombos * 0.01f );
		}

		m_iCombos= 0;
		m_iStones= 0;
		//add mun

	}

	if( m_ActiveFields.size() == 0 )
	{
		return;
	}

	m_iTimeToCombo-= c_iTimeStep;
	if( m_iTimeToCombo > 0 )
	{
		return;
	}

	for( int i= 0; i < (int)m_ActiveFields.size(); ++i )
	{
		int x= m_ActiveFields[ i ].m_iX;
		int y= m_ActiveFields[ i ].m_iY;
		if( fields[x][y] != NULL )
		{
			std::set< SHint > setFields;
			setFields.insert( m_ActiveFields[ i ] );
			checkNeighbours( setFields, fields[x][y]->type );

			if( setFields.size() > 3 || fields[x][y]->type == 7)
			{
				m_iCombos++;
				m_iStones+= (int)setFields.size();
				m_PointsX.m_fValue+= g_hge->Random_Float( -10.0f, 10.0f );
				m_PointsY.m_fValue+= g_hge->Random_Float( -10.0f, 10.0f );
				m_iResetCombo= 3 * c_iComboTime;
				cx = cy = 0.0f;

				for( std::set< SHint >::iterator it= setFields.begin();
					it != setFields.end(); ++it )
				{
					int x= it->m_iX;
					int y= it->m_iY;
					DWORD color = g_sprFieldBlock[fields[x][y]->type]->GetColor();
					cx+=x;cy+=y;
					delete fields[x][y];
					fields[x][y]= NULL;

					for( int i = 0; i < 15; ++i )
					{
						EffectSprite* pNew= new EffectSprite( *this );
						pNew->x= 96.0f + g_fBlockWidth * x;// + g_hge->Random_Float( -10.0f, 10.0f );
						pNew->y= g_fBlockHeight * y;
						pNew->rot=0.0f;

						pNew->dx= g_hge->Random_Float( -0.9f, 0.9f );
						pNew->dy= -0.5f + g_hge->Random_Float( -0.3f, 0.3f );
						pNew->drot= pNew->dx * 0.1f;
						pNew->g= 0.01f;
						pNew->age= 0.0f;
						pNew->maxage= 3.0f;

						pNew->scale= g_hge->Random_Float( 0.1f, 0.6f );

						pNew->sprSprite= g_sprStarEffect;
						pNew->color.SetHWColor(color);

						adoptObject( pNew );
					}
				}

				cx /= setFields.size();
				cy /= setFields.size();

				int pan = calcPan(64.0f + 64.0f * cx);
				g_hge->Effect_PlayEx(g_sndBlock, 100, pan, g_hge->Random_Float(0.8f, 1.2f));
			}
		}
	}

	m_ActiveFields.clear();
	crush();
}

void Level::checkNeighbours( std::set< SHint >& hints, int iType )
{
	while( true )
	{
		bool bNew= false;
		for( std::set< SHint >::iterator it= hints.begin();
				it != hints.end(); ++it )
		{
			for( int i= 0; i < 4; ++i )
			{
				int xx[]= {0,1,0,-1};
				int yy[]= {1,0,-1,0};
				int x= xx[ i ];
				int y= yy[ i ];
				
				x+= it->m_iX;
				y+= it->m_iY;

				if( x >= 0 && x < LEVEL_WIDTH &&
					y >= 0 && y < LEVEL_HEIGHT )
				{
					if( fields[x][y] != NULL )
					{
						if( fields[x][y]->type == iType )
						{
							SHint newHint;
							newHint.m_iX= x;
							newHint.m_iY= y;
							std::set< SHint >::iterator itNew= hints.find( newHint );
							if( itNew == hints.end() )
							{
								hints.insert( newHint );
								bNew= true;
								break;
							}
						}
					}
				}
			}
		}
		if( !bNew )
		{
			return;
		}
	}
}

void Level::crush()
{
	for(int x = 0; x < LEVEL_WIDTH; x++)
	{
		for(int y = 1; y < LEVEL_HEIGHT; y++)
		{
			if( fields[x][y] != 0 )
			{
				if( fields[x][y - 1] == 0 )
				{
					int yy = y - 1;
					for(; yy > 1; yy--)
					{
						if( fields[x][yy - 1] != 0 )
						{
							break;
						}
					}

					fields[x][yy]= fields[x][y]; 
					fields[x][y]= NULL;
					fields[x][yy]->m_tvY.m_fValue= (y - yy) * g_fBlockHeight;

					SHint hint;
					hint.m_iX= x;
					hint.m_iY= yy;

					m_ActiveFields.push_back( hint );
					m_iTimeToCombo= c_iComboTime / 2;
					m_iResetCombo= 3 * c_iComboTime;
				}
			}
		}
	}
}

void Level::StartGirl( float fInten )
{
	m_fGirlTime= 10.0f;
	m_iGirlNumber= rand() % 5;
	switch( rand() % 15 )
	{
	case 1:
		m_GirlText= "I'm impressed by your GREAT skills.";
		break;
	case 2:
		m_GirlText= "Teeheehee, more of it please!";
		break;
	case 3:
		m_GirlText= "I love good combos!";
		break;
	case 4:
		m_GirlText= "Kill them all, just for me!";
		break;
	case 5:
		m_GirlText= "You make me very proud!";
		break;
	case 6:
		m_GirlText= "Awesome!";
		break;
	case 7:
		m_GirlText= "Keep up the good work!";
		break;
	case 8:
		m_GirlText= "I'm a little princess, heehee!";
		m_iGirlNumber= 1;
		break;
	case 9:
		m_GirlText= "Wonderful, just wonderful! And now go on.";
		break;
	case 10:
		m_GirlText= "This GAME is superfun!";
		break;
	case 11:
		m_GirlText= "This was so beautiful!";
		break;
	case 12:
		m_GirlText ="I admire you so much!";
		break;
	case 13:
		m_GirlText="It pleases me to see you play!";
		break;
	case 14:
		m_GirlText="I could watch you all day!";
		break;
	case 0:
		m_GirlText= "Oh my CODER, you're my hero!";
		break;
	}
	m_fGirlIntensity= fInten;
}