#include "PCH.h"
#include "PlayerCannon.h"
#include "Level.h"
#include "gfx.h"
#include "sfx.h"
#include "Rocket.h"

PlayerCannon::PlayerCannon(Level& level)
	: GameObject(level), mx(512.0f), my(384.0f), crosshairAngle(0.0f)
{
	gunX[0] = 64.0f;
	gunX[1] = 13.0f * 64.0f;
	gunY = 700.0f;
	gunAngle[0] = gunAngle[1] = 0.0f;
	ammo[0] = ammo[1] = 10;
	reload[0] = reload[1] = 1.0f;
	ammoAddedFX[0]=ammoAddedFX[1]=0.0f;

	m_iLayer = g_PlayerCannonDepth + 1;
}

PlayerCannon::~PlayerCannon()
{

}

void PlayerCannon::update()
{
	if(g_hge->Input_IsMouseOver())
	{
		g_hge->Input_GetMousePos(&mx, &my);
		if(mx > 14.0f * 64) mx = 14.0f * 64;
		if(my > gunY - 50.0f) my = gunY - 50.0f;
		g_hge->Input_SetMousePos(mx, my);
	}

	int gun = -1;
	if( !level.m_bGameOver )
	{
		if(g_hge->Input_KeyDown(HGEK_LBUTTON)) gun = 0;
		else if(g_hge->Input_KeyDown(HGEK_RBUTTON)) gun = 1;
	}
	
	if(gun != -1 && ammo[gun] && reload[gun] <= 0.0f)
	{
		float vx = sinf(gunAngle[gun]);
		float vy = -cosf(gunAngle[gun]);
		Rocket* rocket = new Rocket(level, gunX[gun] + vx * 25.0f, gunY + vy * 25.0f, vx * 50.0f, vy * 50.0f, mx, my);
		level.adoptObject(rocket);
		ammo[gun]--;
		reload[gun] = 0.5f;

		g_hge->Effect_PlayEx(g_sndFire, 100, gun == 0 ? -50 : 50, g_hge->Random_Float(0.9f, 1.1f));
	}

	if(level.getNumRockets())
	{
		crosshairAngle += 0.01f * 2 * M_PI;
	}
	else
	{
		float targetAngle = crosshairAngle - fmodf(crosshairAngle, M_PI / 2);
		crosshairAngle = 0.9f * crosshairAngle + 0.1f * targetAngle;
	}

	for(int i = 0; i < 2; i++)
	{
		reload[i] -= 0.01f;
		ammoAddedFX[i] -= 0.015f;
		if(ammoAddedFX[i]<0.0f)ammoAddedFX[i]=0.0f;
	}
}

void PlayerCannon::render()
{
	for(int i = 0; i < 2; i++)
	{
		float gx = gunX[i];
		float gy = gunY;
		float dx = mx - gx;
		float dy = my - gy;
		float d = sqrtf(dx*dx + dy*dy);
		dx /= d; dy /= d;
		gunAngle[i] = atan2f(dy, dx) + M_PI / 2;
		g_sprRock->SetFlip(i == 1, false);
		g_sprRock->Render(gunX[i], gunY + 25.0f);
		g_sprCannon->RenderEx(gunX[i], gunY, gunAngle[i]);

		hgeColor color(1.0f, 1.0f, 1.0f, 1.0f);
		if(ammo[i] <= 3) color = hgeColor(1.0f, 1.0f, 0.0f, 1.0f);
		if(ammo[i] == 0) color = hgeColor(1.0f, 0.0f, 0.0f, 1.0f);
		float scale = 1.0f;
		if(ammoAddedFX[i] > 0.0f) scale = 1.0f + 2.5f * sinf(ammoAddedFX[i] * M_PI);
		g_fntSmall->SetColor(color.GetHWColor());
		g_fntSmall->SetScale(scale);
		char bla[1024];
		sprintf_s( bla, "%d", ammo[i] );
		g_fntSmall->Render(gx, gy + 24.0f, HGETEXT_CENTER | HGETEXT_MIDDLE, bla );
	}

	g_sprCrosshair->RenderEx(mx, my, crosshairAngle);
}

void PlayerCannon::addAmmo(int num)
{
	int oldAmmo[] = {ammo[0],ammo[1]};
	while(num)
	{
		if(ammo[0] < ammo[1]) ammo[0]++;
		else if(ammo[0] > ammo[1]) ammo[1]++;
		else ammo[g_hge->Random_Int(0, 1)]++;
		num--;
	}

	for(int i=0;i<2;i++)
	{
		ammoAddedFX[i]=ammo[i]==oldAmmo[i] ? 0.0f : 1.0f;
	}
}