#include "PCH.h"
#include "Rocket.h"
#include "Explosion.h"
#include "gfx.h"
#include "Level.h"

Rocket::Rocket(Level& level, float x, float y, float vx, float vy, float tx, float ty)
	: GameObject(level), x(x), y(y), vx(vx), vy(vy), tx(tx), ty(ty), age(0.0f), oldTargetDist(FLT_MAX)
{
	particleSys = 0;

	m_iLayer = g_PlayerCannonDepth;
}

Rocket::~Rocket()
{
	if(particleSys) level.getParticleMgr()->KillPS(particleSys);
}

void Rocket::update()
{
	float dx = x - tx;
	float dy = y - ty;
	float targetDist = dx*dx + dy*dy;

	if(targetDist > oldTargetDist)
	{
		// Rakete entfernt sich vom Ziel und muss explodieren
		dead = true;
		Explosion* explosion = new Explosion(level, tx, ty);
		level.adoptObject(explosion);
	}

	oldTargetDist = targetDist;

	x += vx * 0.01f;
	y += vy * 0.01f;
	vx += g_hge->Random_Float(-0.05f, 0.05f);
	vy += g_hge->Random_Float(-0.05f, 0.05f);
	vx *= 1.015f;
	vy *= 1.015f;
	age += 0.01f;

	if(x < -50.0f || y < -50.0f || x > 1024.0f + 50 || y > 768.f + 50) dead = true;

	if(age > 0.3f && !particleSys)
	{
		hgeParticleSystemInfo *psi = (hgeParticleSystemInfo*)g_hge->Resource_Load("../data/trail.psi");
		psi->sprite = g_sprFire;
		psi->nEmission = 250;
		psi->fSizeStart = 0.25f;
		psi->fSizeEnd = 0.0f;
		psi->colColorStart=hgeColor(1.0f,1.0f,0.0f,0.25f);
		psi->colColorEnd=hgeColor(0.0f,0.0f,1.0f,0.0f);
		particleSys = level.getParticleMgr()->SpawnPS(psi, 0.0f, 0.0f);
	}

	if(particleSys) particleSys->MoveTo(x, y);
}

void Rocket::render()
{
	float v = sqrtf(vx*vx + vy*vy);
	float dx = vx/v;
	float dy = vy/v;
	float angle = atan2f(dy, dx) + M_PI / 2;
	int state = 0;
	if(age > 0.3f) state = 1;
	if(age > 0.6f) state = 2;
	g_sprRocket[state]->RenderEx(x, y, angle);
}